/******************************************************************************
 **																			 **
 ** 	Module:		AttributeSet_Lib.c										 **
 ** 																		 **
 ** 																		 **
 ** 	Purpose: 	Typed attributeSet routines		 						 **
 ** 																		 **
 ** 																		 **
 ** 	Copyright (C) 1992, 1993, 1994										 **
 **			Apple Computer, Inc.  All rights reserved.						 **
 ** 																		 **
 ** 	Change Log:															 **
 **		02/25/94	kmd	created												 **
 ** 																		 **
 *****************************************************************************/
#include "QD3D.h"
#include "AttributeSet_Lib.h"

/******************************************************************************
 **																			 **
 **							AttributeSet Typed Routines						 **
 **																			 **
 *****************************************************************************/

TQ3Status AttributeSet_AddSurfaceUV(
	TQ3AttributeSet		attributeSet,
	const TQ3Param2D		*data)
{
	return
		Q3AttributeSet_Add(
			attributeSet, 
			kQ3AttributeTypeSurfaceUV, 
			(const void *) data);
}

TQ3Status AttributeSet_AddShadingUV(
	TQ3AttributeSet		attributeSet,
	const TQ3Param2D		*data)
{
	return
		Q3AttributeSet_Add(
			attributeSet, 
			kQ3AttributeTypeShadingUV, 
			(const void *) data);
}

TQ3Status AttributeSet_AddNormal(
	TQ3AttributeSet		attributeSet,
	const TQ3Vector3D	*data)
{
	return
		Q3AttributeSet_Add(
			attributeSet, 
			kQ3AttributeTypeNormal, 
			(const void *) data);
}

TQ3Status AttributeSet_AddAmbientCoefficient(
	TQ3AttributeSet		attributeSet,
	const float			*data)
{
	return
		Q3AttributeSet_Add(
			attributeSet, 
			kQ3AttributeTypeAmbientCoefficient, 
			(const void *) data);
}

TQ3Status AttributeSet_AddDiffuseColor(
	TQ3AttributeSet		attributeSet,
	const TQ3ColorRGB		*data)
{
	return
		Q3AttributeSet_Add(
			attributeSet, 
			kQ3AttributeTypeDiffuseColor, 
			(const void *) data);
}

TQ3Status AttributeSet_AddSpecularColor(
	TQ3AttributeSet		attributeSet,
	const TQ3ColorRGB		*data)
{
	return
		Q3AttributeSet_Add(
			attributeSet, 
			kQ3AttributeTypeSpecularColor, 
			(const void *) data);
}

TQ3Status AttributeSet_AddSpecularControl(
	TQ3AttributeSet		attributeSet,
	const float			*data)
{
	return
		Q3AttributeSet_Add(
			attributeSet, 
			kQ3AttributeTypeSpecularControl, 
			(const void *) data);
}

TQ3Status AttributeSet_AddTransparencyColor(
	TQ3AttributeSet		attributeSet,
	const TQ3ColorRGB		*data)
{
	return
		Q3AttributeSet_Add(
			attributeSet, 
			kQ3AttributeTypeTransparencyColor, 
			(const void *) data);
}

TQ3Status AttributeSet_AddSurfaceTangent(
	TQ3AttributeSet		attributeSet,
	const TQ3Tangent2D	*data)
{
	return
		Q3AttributeSet_Add(
			attributeSet, 
			kQ3AttributeTypeSurfaceTangent, 
			(void *) data);
}

TQ3Status AttributeSet_AddSurfaceShader(
	TQ3AttributeSet			attributeSet,
	TQ3SurfaceShaderObject	*data)
{
	return
		Q3AttributeSet_Add(
			attributeSet, 
			kQ3AttributeTypeSurfaceShader, 
			(void *) data);
}


TQ3Status AttributeSet_GetSurfaceUV(
	TQ3AttributeSet		attributeSet,
	TQ3Param2D			*data)
{
	return
		Q3AttributeSet_Get(
			attributeSet, 
			kQ3AttributeTypeSurfaceUV, 
			(void *) data);
}

TQ3Status AttributeSet_GetShadingUV(
	TQ3AttributeSet		attributeSet,
	TQ3Param2D			*data)
{
	return
		Q3AttributeSet_Get(
			attributeSet, 
			kQ3AttributeTypeShadingUV, 
			(void *) data);
}

TQ3Status AttributeSet_GetNormal(
	TQ3AttributeSet		attributeSet,
	TQ3Vector3D			*data)
{
	return
		Q3AttributeSet_Get(
			attributeSet, 
			kQ3AttributeTypeNormal, 
			(void *) data);
}

TQ3Status AttributeSet_GetAmbientCoefficient(
	TQ3AttributeSet		attributeSet,
	float				*data)
{
	return
		Q3AttributeSet_Get(
			attributeSet, 
			kQ3AttributeTypeAmbientCoefficient, 
			(void *) data);
}

TQ3Status AttributeSet_GetDiffuseColor(
	TQ3AttributeSet		attributeSet,
	TQ3ColorRGB			*data)
{
	return
		Q3AttributeSet_Get(
			attributeSet, 
			kQ3AttributeTypeDiffuseColor, 
			(void *) data);
}

TQ3Status AttributeSet_GetSpecularColor(
	TQ3AttributeSet		attributeSet,
	TQ3ColorRGB			*data)
{
	return
		Q3AttributeSet_Get(
			attributeSet, 
			kQ3AttributeTypeSpecularColor, 
			(void *) data);
}

TQ3Status AttributeSet_GetSpecularControl(
	TQ3AttributeSet		attributeSet,
	float				*data)
{
	return
		Q3AttributeSet_Get(
			attributeSet, 
			kQ3AttributeTypeSpecularControl, 
			(void *) data);
}

TQ3Status AttributeSet_GetTransparencyColor(
	TQ3AttributeSet		attributeSet,
	TQ3ColorRGB			*data)
{
	return
		Q3AttributeSet_Get(
			attributeSet, 
			kQ3AttributeTypeTransparencyColor, 
			(void *) data);
}

TQ3Status AttributeSet_GetSurfaceTangent(
	TQ3AttributeSet		attributeSet,
	TQ3Tangent2D			*data)
{
	return
		Q3AttributeSet_Get(
			attributeSet, 
			kQ3AttributeTypeSurfaceTangent, 
			(void *) data);
}

TQ3Status AttributeSet_GetSurfaceShader(
	TQ3AttributeSet			attributeSet,
	TQ3SurfaceShaderObject	*data)
{
	return
		Q3AttributeSet_Get(
			attributeSet, 
			kQ3AttributeTypeSurfaceShader, 
			(void *) data);
}
